/* File: anagram-clusters.cpp
 *
 * A program to find the largest anagram cluster in the English language.
 */
#include <iostream>
#include <string>
#include "map.h"
#include "set.h"
#include "lexicon.h"
#include "simpio.h"
#include "strlib.h"
using namespace std;

/* Given a string, returns a sorted version of that string. */
string sort(string& input);

int main() {
	Lexicon words("EnglishWords.dat");
	Map<string, Set<string> > clusters;
	
	/* Distribute each word into the appropriate cluster. */
	foreach (string word in words) {
		/* Note that this use of [] does one of two different things:
		 * 1. If the cluster exists, it adds the word to that cluster.
		 * 2. If the cluster doesn't exist, it's created, and the word is added.
		 */
		clusters[sort(word)] += word;
	}
	
	while (true) {
		/* Read a word from the user. */
		string word = toLowerCase(getLine("Enter a word: "));
		if (word == "") break;
		
		/* If we know that word, output all anagrams of that word. */
		if (clusters.containsKey(sort(word))) {
			foreach (string anagram in clusters[sort(word)]) {
				cout << anagram << endl;
			}
		}
		/* Otherwise, report an error. */
		else {
			cout << "Sorry, I don't know that word." << endl;
		}
	}
		
	return 0;
}

/* Sorts the letters of a string into ascending order using the
 * counting sort algorithm.
 */
string sort(string& input) {
	/* Build up a frequency table of the letters in the word. */
	Map<char, int> frequencies;
	foreach (char ch in input) {
		frequencies[ch]++;
	}
	
	/* Reconstruct the string, in sorted order, by appending
	 * the appropriate number of copies of each character to
	 * a resulting string.
	 */
	string result;
	foreach (char ch in frequencies) {
		for (int i = 0; i < frequencies[ch]; i++) {
			result += ch;
		}
	}
	return result;	
}







		
	
